:reset
import org.apache.spark.ml.evaluation.RegressionEvaluator
import org.apache.spark.ml.regression.{DecisionTreeRegressionModel,DecisionTreeRegressor}
import org.apache.spark.ml.regression.{RandomForestRegressionModel, RandomForestRegressor}
import org.apache.spark.ml.regression.{GBTRegressionModel, GBTRegressor}
import org.apache.spark.ml.linalg.Vectors
import org.apache.spark.ml.feature.LabeledPoint
import org.apache.spark.rdd.RDD
import org.apache.spark.sql.DataFrame
import scala.collection.mutable.ArrayBuffer
import spark.implicits._


// Número de cada atributo
val ATTR_time = 0
val ATTR_month = 1
val ATTR_hour = 2
val ATTR_demand = 3
val ATTR_pressure = 4
val ATTR_work_day = 5
val ATTR_season = 6
val ATTR_date = 7

val DATA_TRAINING = "training"
val DATA_TEST = "test"

// Nombre de los modelos
val DECISION_TREE = "DecisionTreeRegressor"
val RANDOM_FOREST = "RandomForestRegressor"
val GBT = "GBTRegressor"

// Parametros para los modelos
var MAXBINS = 24
val NUM_TREES = 60

/***************************************************
******************** FUNCIONES *********************
****************************************************/

/* Función que lee los archivos de entrenamiento y prueba y los transforma a RDD */
def readData(): (RDD[Array[String]], RDD[Array[String]]) = {
  // Leemos datos y creamos RDD de String
  var data_training = sc.textFile(DATA_TRAINING)
  var data_test_raw = sc.textFile(DATA_TEST)

  //Dividimos por comas
  var data:RDD[Array[String]] =  data_training.map(line => line.split (";"))
  var dataTest:RDD[Array[String]] =  data_test_raw.map(line => line.split (";"))
  
  (data, dataTest)
}

/* Función que transforma las características a codificación binaria y los atributos numéricos a Double */
def get_features (row: Array[String]): Array[Double] = {
  var attr = 0
  var cat_vec = Array[Double]()
  var num_vec = Array[Double]()

  // Transform the numerical features to double
  for (attr <- 1 to 2){
    cat_vec = cat_vec :+ row(attr).toDouble
  }
  for (attr <- 5 to 6){
    num_vec = num_vec :+ row(attr).toDouble
  }
// Concatenate the two array of features
  cat_vec ++ num_vec
//var attr = 0
}

/* Función que devuelve el RDD de LabeledPoint a partir del RDD de Array de String */
def getDataFrame(data: RDD[Array[String]]): DataFrame = {
    data.map{row => LabeledPoint(
      (row(ATTR_demand).toDouble).toDouble, 
      Vectors.dense(get_features(row)))
    }.toDF("label", "features")
}


/***************************************************
*****************  FUNCIONES EN TEST ***************
****************************************************/

/* Función que devuelve el DataFrame con las predicciones y las labels de un modelo DecisionTreeRegressor */
def getPredictionsAndLabelsDT(testCV: RDD[Array[String]], model: DecisionTreeRegressionModel): DataFrame = {
    // Hacer predicciones.
    var predictionAndLabels = model.transform(getDataFrame(testCV))
    // Obtener la label y la prediction de los ejemplos de prueba
    predictionAndLabels.select("prediction", "label")

}

/* Función que devuelve el DataFrame con las predicciones y las labels de un modelo RandomForestRegressor */
def getPredictionsAndLabelsRF(testCV: RDD[Array[String]], model: RandomForestRegressionModel): DataFrame = {
  // Hacer predicciones.
  var predictionAndLabels = model.transform(getDataFrame(testCV))
  // Obtener la label y la prediction de los ejemplos de prueba
  predictionAndLabels.select("prediction", "label")

}
/* Función que devuelve el RDD con las predicciones y las labels de un modelo GBTRegressor */
def getPredictionsAndLabelsGBT(testCV: RDD[Array[String]], model: GBTRegressionModel): DataFrame = {
    // Hacer predicciones.
    var predictionAndLabels = model.transform(getDataFrame(testCV))
    // Obtener la label y la prediction de los ejemplos de prueba
    predictionAndLabels.select("prediction", "label")
}

/* Función que obtiene las metricas para la evaluacion */
def getRegressionEvaluator(predictionsAndLabels: DataFrame): (Double, Double, Double, Double) = {
    
    var rmse = new RegressionEvaluator().setMetricName("rmse").evaluate(predictionsAndLabels);
    var mse = new RegressionEvaluator().setMetricName("mse").evaluate(predictionsAndLabels);
    var mae = new RegressionEvaluator().setMetricName("mae").evaluate(predictionsAndLabels);
    var r2 = new RegressionEvaluator().setMetricName("r2").evaluate(predictionsAndLabels);

    (rmse, mse, mae, r2)
}


/***************************************************
**************** EVALUACIÓN MODELO *****************
****************************************************/

// Leemos los datos
var (data, dataTest) = readData()
//veo que es lo que hay en este DF de test
getDataFrame(dataTest).show

//Leemos los modelos almacenados
val modelDT = DecisionTreeRegressionModel.load("Modelo/"+ DECISION_TREE)
val modelRF = RandomForestRegressionModel.load("Modelo/" + RANDOM_FOREST)
val modelGBT = GBTRegressionModel.load("Modelo/" + GBT)

// Obtenemos las predicciones y las labels
val predictionsAndLabelsDT = getPredictionsAndLabelsDT(dataTest, modelDT)
val predictionsAndLabelsRF = getPredictionsAndLabelsRF(dataTest, modelRF)
val predictionsAndLabelsGBT = getPredictionsAndLabelsGBT(dataTest, modelGBT) 

// Obtenemos las métricas
val (rmseDT, mseDT, maeDT, r2DT) = getRegressionEvaluator(predictionsAndLabelsDT)
val (rmseRF, mseRF, maeRF, r2RF) = getRegressionEvaluator(predictionsAndLabelsRF)
val (rmseGBT, mseGBT, maeGBT, r2GBT) = getRegressionEvaluator(predictionsAndLabelsGBT)


println("\n--------------- DECISION TREE REGRESSION  --------------- ")
println("Coeficiente de determinación: " + r2DT + "  Error cuadrático medio: " + mseDT)
println("Raiz del error cuadrático medio: " + rmseDT + "  Error absoluto medio: " + maeDT  )

predictionsAndLabelsDT.show(7)

println("\n--------------- RANDOM FOREST REGRESSION  --------------- ")
println("Coeficiente de determinación: " + r2RF + "  Error cuadrático medio: " + mseRF)
println("Raiz del error cuadrático medio: " + rmseRF + "  Error absoluto medio : " + maeRF )

predictionsAndLabelsRF.show(7)

println("\n--------------- GRADIENT-BOOSTED TREE REGRESSION --------------- ")
println("Coeficiente de determinación: " + r2GBT + "  Error cuadrático medio: " + mseGBT )
println("Raiz del error cuadrático medio: " + rmseGBT + "  Error absoluto medio: " + maeRF )

predictionsAndLabelsGBT.show(7)

predictionsAndLabelsDT.repartition(1).write.format("com.databricks.spark.csv").option("header", "true").save("output/"+DECISION_TREE)
predictionsAndLabelsRF.repartition(1).write.format("com.databricks.spark.csv").option("header", "true").save("output/"+RANDOM_FOREST)
predictionsAndLabelsGBT.repartition(1).write.format("com.databricks.spark.csv").option("header", "true").save("output/"+GBT)
